import io
import logging
import pathlib
import re
import sys
import types

from IPython import get_ipython
from IPython.core.interactiveshell import InteractiveShell
from nbformat import read

log = logging.getLogger(__name__)


def find_imports(code: str):
    """Match imports

    This code matches
    >>> import abc
    >>> from abc import ABC
    for the beginning of a new line
    """
    return re.search(r"(^|\r|\n|\r\n)(import|from)", code)


def find_functions(code: str):
    """Match functions

    This code matches everything that contains "def"
    """
    return re.search("def ", code)


def find_classes(code: str):
    """Match classes

    This  code matches everything that contains "class"
    """
    return re.search("class ", code)


def find_decorators(code: str):
    """Match decorators

    This code matches everything that contains @<str>
    """
    return re.search(r"@[a-zA-Z]+", code)


class NotebookLoader:
    """Module Loader for Jupyter Notebooks

    References
    ----------
    Modified version of the example from
    jupyter-notebook.readthedocs.io/en/stable/examples/Notebook/Importing%20Notebooks.html

    """

    def __init__(self, file):
        self.shell = InteractiveShell.instance()
        self.file = pathlib.Path(file).with_suffix(".ipynb")

    def load_module(
        self,
        run_imports=True,
        run_functions=True,
        run_classes=True,
        run_decorators=True,
        run_all=False,
    ):
        """import a notebook as a module

        Attributes
        ----------
        run_imports: bool, default=True
            Use exec on all matched import statements
        run_functions: bool, default=True
            Use exec on all matched function definitions
        run_classes: bool, default=True
            Use exec on all matched class defintions
        run_decorators:  bool, default=True
            Use exec on all matched decrorators
        run_all: bool, default=False
            Ignore regex and run everything
        """

        log.debug(f"importing Jupyter notebook from {self.file}")

        # load the notebook object
        with io.open(self.file, "r", encoding="utf-8") as f:
            nb = read(f, 4)

        # create the module and add it to sys.modules
        # if name in sys.modules:
        #    return sys.modules[name]
        mod = types.ModuleType(self.file.stem)
        mod.__file__ = self.file.as_posix()
        mod.__loader__ = self
        mod.__dict__["get_ipython"] = get_ipython
        sys.modules[self.file.stem] = mod

        # extra work to ensure that magics that would affect the user_ns
        # actually affect the notebook module's ns
        save_user_ns = self.shell.user_ns
        self.shell.user_ns = mod.__dict__

        try:
            for cell in nb.cells:
                if cell.cell_type == "code":
                    # transform the input to executable Python
                    code = self.shell.input_transformer_manager.transform_cell(
                        cell.source
                    )
                    run_code = run_all
                    if run_imports and find_imports(code):
                        log.debug(f"Found import statement in \n{code}")
                        run_code = True
                    if run_functions and find_functions(code):
                        log.debug(f"Found function definition in \n{code}")
                        run_code = True
                    if run_classes and find_classes(code):
                        log.debug(f"Found class definition in \n{code}")
                        run_code = True
                    if run_decorators and find_decorators(code):
                        log.debug(f"Found decorator in \n{code}")
                        run_code = True

                    if run_code:
                        # run the code in the module to get e.g. all imports correctly
                        exec(code, mod.__dict__)
        finally:
            self.shell.user_ns = save_user_ns
        return mod



import logging
import sys

from znipy.core.loader import NotebookLoader

__all__ = [NotebookLoader.__name__]

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

# Formatter for advanced logging
# formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
formatter = logging.Formatter("%(asctime)s %(module)s (%(levelname)s): %(message)s")

channel = logging.StreamHandler(sys.stdout)
channel.setLevel(logging.DEBUG)
channel.setFormatter(formatter)

logger.addHandler(channel)



import logging

log = logging.getLogger(__name__)
import pathlib
import re
import subprocess


def jupyter_class_to_file(silent, nb_name, module_name, nb_class_path):
    """Extract the class definition form an ipynb file"""

    nb_class_path = pathlib.Path(nb_class_path)

    if not silent:
        log.warning(
            "Jupyter support is an experimental feature! Please save your "
            "notebook before running this command!\n"
            "Submit issues to https://github.com/zincware/ZnTrack."
        )
        log.warning(f"Converting {nb_name} to file {module_name}.py")

    nb_name = pathlib.Path(nb_name)

    subprocess.run(
        ["jupyter", "nbconvert", "--to", "script", nb_name],
        capture_output=not silent,
    )

    reading_class = False

    imports = ""

    class_definition = ""

    with pathlib.Path(nb_name).with_suffix(".py").open("r") as f:
        for line in f:
            if line.startswith("import") or line.startswith("from"):
                imports += line
            if reading_class:
                if (
                    re.match(r"\S", line)
                    and not line.startswith("#")
                    and not line.startswith("class")
                    and not line.startswith("def")
                    and not line.startswith("@")
                    and not line.startswith(")")
                ):
                    reading_class = False
            if reading_class or line.startswith("class"):
                reading_class = True
                class_definition += line
            if line.startswith("@"):  # handle decorators
                reading_class = True
                class_definition += line

    src = imports + "\n\n" + class_definition

    src_file = pathlib.Path(nb_class_path, module_name).with_suffix(".py")
    nb_class_path.mkdir(exist_ok=True, parents=True)

    src_file.write_text(src)

    # Remove converted ipynb file
    nb_name.with_suffix(".py").unlink()



import pathlib

import setuptools

long_description = pathlib.Path("README.md").read_text()
required_packages = pathlib.Path("requirements.txt").read_text().splitlines()

setuptools.setup(
    name="znipy",
    version="0.1.0",
    author="zincwarecode",
    author_email="zincwarecode@gmail.com",
    description="Easy imports from Jupyter Notebooks",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/zincware/ZnIPy",
    download_url="https://github.com/zincware/ZnIPy/archive/beta.tar.gz",
    keywords=["IPython", "Jupyter", "ZnTrack"],
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Eclipse Public License 2.0 (EPL-2.0)",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
    install_requires=required_packages,
)
